#!/bin/bash
#$ -l h_rt=72:00:00
#$ -cwd
#$ -N jobname
#$ -pe mpich 64
#$ -V
#$ -j yes

#source $HOME/bin/change_ld_path.sh 

check_time_entry () {
	delta_t=0.1
	wall_clock_time=187200
	
	output_file=$1
	nodes_uniq=$2
	tmp_dir=$3
	control_dict='system/controlDict'

	new_config="blub"
	echo sleeping ${wall_clock_time} seconds
	sleep ${wall_clock_time}
	current_time=`grep "^Time = " ${output_file} | tail -n 1 | cut -d '=' -f 2| sed -e 's/ //'`
	echo found simulation time after waiting to be ${current_time}

	#end_time=`expr ${current_time} + ${delta_t}`
	end_time=`echo "${current_time} + ${delta_t}" |bc`
	echo setting end time to ${end_time}

	write_interval_old="writeInterval   50;"
	write_interval_new="writeInterval   1;"

	end_time_old="endTime         8500.0;"
	end_time_new="endTime         ${end_time};"

	cat ${control_dict}  | sed -e "s/${write_interval_old}/${write_interval_new}/" |
		sed -e "s/${end_time_old}/${end_time_new}/" > $new_config

	for i in ${nodes_uniq}
	do
		echo copying new dict to $i
		scp -p ${new_config} $i:${tmp_dir}/${control_dict}
	done
	cp ${new_config} /tmp
	echo finished checking for runtime
}









################################################
#####         start copying process         ####
################################################



#temporary directory on nodes:
#tmp_dir="/work/tmp/aram_job_${JOB_ID}"
tmp_dir="/work/tmp/aram_job_${JOB_ID}/OBB_arcedU0502"
tmp_dir_decompose="/work/tmp/aram_job_${JOB_ID}"



#naming of the tar files that should be distributed to
#the nodes. Each tar file should contain 8 subdirs.

#file names are completed by appending a number without leading
#zeros + tar.gz
process_tar_file_base="processes_"

#naming of the outputfiles; number and tar.gz are appended automatically
output_tar_base="output_"

#save output data to this directory
start_dir=`pwd`
data_save="${start_dir}/data_${JOB_ID}"


output_file=${start_dir}/${JOB_NAME}.o${JOB_ID}
echo having outpufile: ${output_file}



nodes_uniq=$(cat $TMPDIR/machines| uniq)


#copy files per node
#tar.gz files contain 8 subdirectories for each process

j=0

for i in $nodes_uniq
    do
    tar_file_name="${process_tar_file_base}${j}.tar.gz"
    echo "creating ${tmp_dir} on $i"
    ssh $i mkdir -p $tmp_dir
    echo "copying $tar_file_name to node $i"
    #scp ${start_dir}\/${tar_file_name} $i:${tmp_dir}
    ssh $i cp ${start_dir}\/${tar_file_name} ${tmp_dir}
    ssh $i cp -r ${start_dir}\/tmp_dictionaries\/* ${tmp_dir}
    
    #adjust decomposeParDict    
    tmp_string=`echo $tmp_dir_decompose |sed -e "s?\/?\\\\\\\/?g"`
    ssh $i "cat ${tmp_dir}\/system/decomposeParDict  |sed -e "'"'"s?/work/tmp/aram_job?${tmp_string}?"'"'" > ${tmp_dir}\/system/decomposeParDict_tmp"
    ssh $i "mv ${tmp_dir}\/system/decomposeParDict_tmp  ${tmp_dir}\/system/decomposeParDict "


    echo "extracting file"
    ssh $i "cd ${tmp_dir} ;tar -zxf ${tmp_dir}\/${tar_file_name}"

    j=$(echo "$j+1"|bc -l)
    done

#mv process tar files for first job  to save directory
if [ ! -d ${start_dir}"/data_00000_start" ]; then
	mkdir -p ${start_dir}"/data_00000_start"
	mv ${process_tar_file_base}* ${start_dir}"/data_00000_start"
else 
	rm ${process_tar_file_base}*
fi

cd ${tmp_dir}
check_time_entry ${output_file} "${nodes_uniq}" ${tmp_dir} &
time mpirun -mca btl_openib_ib_timeout 20 -machinefile $TMPDIR/machines -np 64 $1 -parallel

#cp files per node back to start directory of job
echo "================================================="
j=0
for i in $nodes_uniq
    do
    tar_file_name="${process_tar_file_base}${j}.tar.gz"
    output_tar_file="${output_tar_base}${j}.tar.gz"
    echo "creating ${output_tar_file} on node $i"
    echo "ssh $i \" cd ${tmp_dir} ;tar -zcf ${output_tar_file} processor*\""
    ssh $i " cd ${tmp_dir} ;tar -zcf ${output_tar_file} processor*"

    echo "copying file back"
    mkdir -p ${data_save}
    #scp  $i:${tmp_dir}\/${output_tar_file} ${data_save}
    ssh $i cp  ${tmp_dir}\/${output_tar_file} ${data_save}
    
    echo "creating links for next job"
    
    ln -s ${data_save}/${output_tar_file} ${start_dir}/${tar_file_name}
    echo "finished creating link"

    #echo "extracting ${output_tar_file}"
    #tar -zxf ${output_tar_file}
    #cleaning up
    #echo "cleaning up $tmp_dir"
    #ssh $i /bin/rm -rf ${tmp_dir}
    j=$(echo "$j+1"|bc -l)
    done

# ----------------------------------------------------------------- end-of-file

